#!/usr/bin/python

import sys, optparse, textwrap, itertools, numpy, os

def main():
    description = "TS-TROSY-NOESY pathway separation"
    usage = "\n\n       %prog [options]\n\n" + \
            "  This script produces two ouput files, one for each pathway.\n"+ \
            "  They are named ser_N and ser_C for Bruker data or\n" + \
            "  acqfil/fid_C and acqfil/fid_N for Varian data.\n" + \
            "  Each file can then be treated as its own ser/fid file.\n" + \
            "  Use the -n option for NUS data."
    opt = optparse.OptionParser(description=description, usage=usage)
    opt.add_option("-n", dest="NUS", metavar="POINTS", default=None, 
                   help="Points in NUS schedule")
   
    options, arguments = opt.parse_args()
    
    if os.path.isfile('procpar') and not os.path.isfile('acqus'):
        varian = True
        bruker = False
        inFilename = 'acqfil/fid'
        outPrefix = 'acqfil/fid'
    elif os.path.isfile('acqus') and not os.path.isfile('procpar'):
        bruker = True
        varian = False
        inFilename = 'ser'
        outPrefix = 'ser'
    elif os.path.isfile('acqus') and os.path.isfile('procpar'):
        print("Ambiguous data type. Both procpar and acqus are present.")
        sys.exit()
    else:
        print("Cannot find procpar or acqus.")
        sys.exit()

    if varian:
        with open('procpar','r') as procpar:
            try:
                while True:
                    line = procpar.next()
                    if line[0:3] == 'np ':
                        line = procpar.next()
                        np = int(line.split()[1]) # Total (real + imag)
                    if options.NUS is None:
                        if line[0:3] == 'ni ':
                            line = procpar.next()
                            ni = int(line.split()[1]) # Complex
                        elif line[0:4] == 'ni2 ':
                            line = procpar.next()
                            ni2 = int(line.split()[1]) # Complex
            except(StopIteration):
                pass
        
        if options.NUS is not None:
            ni = 1
            ni2 = int(options.NUS)

        # Define Numpy type for Varian fid file. Varian data is big endian.
        # There is a 32 byte file header.
        # The first int is the number of FIDs in the file.
        # Normally this is ni*ni2*4; in this case it is ni*ni2*16.
        # We will not need to change the rest of the file header.
        # Each FID has a 28 byte block header followed by np four-byte integers.
        dt_in = numpy.dtype( [ ('nblocks','>i4'), ('fileHeader','>i4',(7,)),
                            ('data',
                             [('blkHdr','>i2',(14,)), ('fid','>f4',(np,))],
                             (ni2,ni,2,2,2,2)) ] )

        # Read the data into a Numpy structured array
        inData = numpy.fromfile(inFilename, dtype=dt_in, count=1)
        
        # Define the Varian output file format
        dt_out = numpy.dtype( [ ('nblocks','>i4'), ('fileHeader','>i4',(7,)),
                             ('data',[('blkHdr','>i2',(14,)),
                                      ('fid','>f4',(np,))] ,(ni2,ni,2,2)) ] )

        # Create a new structured array and copy the headers
        outData = numpy.zeros(1, dtype=dt_out)
        # Correct the number of blocks
        outData['nblocks'] = inData['nblocks']/4
        # None of the other file header parameters need to be changed
        outData['fileHeader'] = inData['fileHeader']
        # The block headers are all identical except for a consecutively
        # numbered index value. Copy everything and overwrite the indices
        tempBlockHeader = numpy.zeros([1,ni2,ni,2,2,14], dtype='>i2')
        tempBlockHeader[...,:] = inData['data']['blkHdr'][0,0,0,0,0,0,0,:]
        tempBlockHeader[...,2] = numpy.arange(1, 2*ni*2*ni2+1, dtype='>i2'). \
                                                  reshape((1,ni2,ni,2,2))
        outData['data']['blkHdr'] = tempBlockHeader
        
        # Nitrogen pathway
        E1x = numpy.sum(inData['data']['fid'][0,:,:,0,0,0,:],axis=2)
        E2y = numpy.sum(inData['data']['fid'][0,:,:,0,0,1,:],axis=2)
        E3x = numpy.sum(inData['data']['fid'][0,:,:,0,1,0,:],axis=2)
        E4y = numpy.sum(inData['data']['fid'][0,:,:,0,1,1,:],axis=2)
        E1y = numpy.sum(inData['data']['fid'][0,:,:,1,0,0,:],axis=2)
        E2x = numpy.sum(inData['data']['fid'][0,:,:,1,0,1,:],axis=2)
        E3y = numpy.sum(inData['data']['fid'][0,:,:,1,1,0,:],axis=2)
        E4x = numpy.sum(inData['data']['fid'][0,:,:,1,1,1,:],axis=2)
        outData['data']['fid'][0,:,:,0,0] =  E1x + E2x - E3x + E4x
        outData['data']['fid'][0,:,:,0,1] =  E1y + E2y + E3y - E4y
        outData['data']['fid'][0,:,:,1,0] = -E1y - E2y + E3y - E4y
        outData['data']['fid'][0,:,:,1,1] =  E1x + E2x + E3x - E4x
        outData.tofile(outPrefix + "_N")
        
        # Carbon pathway
        RR = numpy.diff(inData['data']['fid'][0,:,:,0,0,:,:],axis=3)
        RI = numpy.diff(inData['data']['fid'][0,:,:,0,1,:,:],axis=3)
        IR = numpy.diff(inData['data']['fid'][0,:,:,1,0,:,:],axis=3)
        II = numpy.diff(inData['data']['fid'][0,:,:,1,1,:,:],axis=3)
        
        RR = numpy.sum(numpy.squeeze(RR,axis=3),axis=2)
        RI = numpy.squeeze(numpy.diff(numpy.squeeze(RI,axis=3),axis=2),axis=2)
        IR = numpy.squeeze(numpy.diff(numpy.squeeze(IR,axis=3),axis=2),axis=2)
        II = numpy.sum(numpy.squeeze(II,axis=3),axis=2)

        # Overwrite Nitrogen data. Keep the same headers.
        outData['data']['fid'][0,:,:,0,0] =  RR
        outData['data']['fid'][0,:,:,0,1] =  RI
        outData['data']['fid'][0,:,:,1,0] = -IR
        outData['data']['fid'][0,:,:,1,1] = -II
        outData.tofile(outPrefix + "_C")


    elif bruker:
        with open('acqus','r') as acqus:
            for line in acqus:
                if line[0:7] == '##$TD= ':
                    td3 = int(line.split('=')[1]) # Total (real + imag)

        if options.NUS is None:
            with open('acqu2s','r') as acqu2s:
                for line in acqu2s:
                    if line[0:7] == '##$TD= ':
                        td2 = int(line.split('=')[1]) / 16 # Complex
            with open('acqu3s','r') as acqu3s:
                for line in acqu3s:
                    if line[0:7] == '##$TD= ':
                        td1 = int(line.split('=')[1]) # Complex
        else:
            td2 = 1
            td1 = int(options.NUS)
        
        # Define Numpy type describing the Bruker ser file
        dt_in = numpy.dtype( ('i4', (td1,td2,8,2,td3)) )
        # Read the data into a Numpy structured array
        inData = numpy.fromfile(inFilename, dtype=dt_in, count=1)
        
        # Define Numpy type describing the output ser file
        dt_out = numpy.dtype( ('i4', (td1,2,td2,2,td3)) )
        # Initialize the output
        outData = numpy.zeros(1, dtype=dt_out)
        
        # Nitrogen pathway
        E1x = numpy.sum(inData[0,:,:,0,:,:],axis=2)
        E2x = numpy.sum(inData[0,:,:,1,:,:],axis=2)
        E3x = numpy.sum(inData[0,:,:,2,:,:],axis=2)
        E4x = numpy.sum(inData[0,:,:,3,:,:],axis=2)
        E1y = numpy.sum(inData[0,:,:,4,:,:],axis=2)
        E2y = numpy.sum(inData[0,:,:,5,:,:],axis=2)
        E3y = numpy.sum(inData[0,:,:,6,:,:],axis=2)
        E4y = numpy.sum(inData[0,:,:,7,:,:],axis=2)
        outData[0,:,0,:,0,:] =  E1x + E2x - E3x + E4x
        outData[0,:,0,:,1,:] =  E1y + E2y + E3y - E4y
        outData[0,:,1,:,0,:] =  E1y + E2y - E3y + E4y
        outData[0,:,1,:,1,:] = -E1x - E2x - E3x + E4x
        outData.tofile(outPrefix + "_N")
        
        # Carbon pathway
        E1x = numpy.squeeze(numpy.diff(inData[0,:,:,0,:,:],axis=2),axis=2)
        E2x = numpy.squeeze(numpy.diff(inData[0,:,:,1,:,:],axis=2),axis=2)
        E3x = numpy.squeeze(numpy.diff(inData[0,:,:,2,:,:],axis=2),axis=2)
        E4x = numpy.squeeze(numpy.diff(inData[0,:,:,3,:,:],axis=2),axis=2)
        E1y = numpy.squeeze(numpy.diff(inData[0,:,:,4,:,:],axis=2),axis=2)
        E2y = numpy.squeeze(numpy.diff(inData[0,:,:,5,:,:],axis=2),axis=2)
        E3y = numpy.squeeze(numpy.diff(inData[0,:,:,6,:,:],axis=2),axis=2)
        E4y = numpy.squeeze(numpy.diff(inData[0,:,:,7,:,:],axis=2),axis=2)
        outData[0,:,0,:,0,:] =  E1x - E3x
        outData[0,:,0,:,1,:] =  E2y - E4y
        outData[0,:,1,:,0,:] = -E1y + E3y
        outData[0,:,1,:,1,:] =  E2x - E4x
        outData.tofile(outPrefix + "_C")


if __name__ == '__main__':
    main()



